
/* This file is created by the "tcl2c" utility, which is included in
 * most "plus"-patches (e.g. for Tcl7.6 and Tcl8.0). Standalone
 * executables can be made by providing alternative initialization
 * functions which don't read files any more. Sometimes, small
 * adaptations to the original libraries are needed to get the
 * application truly standalone. The "plus"-patches contain these
 * adaptations for Tcl and Tk. If you just create your own
 * Xxx_InitStandAlone() function for your package, you can
 * easyly extend the "tcl2c" utility to your own requirements.
 *
 *	Jan Nijtmans
 *	NICI (Nijmegen Institute of Cognition and Information)
 *	email: nijtmans@nici.kun.nl
 *	url:   http://www.cogsci.kun.nl/~nijtmans/
 */
#include "tcl.h"

/*
 * Defines to replace the standard Xxx_Init calls to Xxx_InitStandAlone.
 * If you don't have this function, just delete the corresponding
 * define such that the normal initialization function is used.
 * Similar: If SafeInit functions exists, you can use these
 * by commenting out the corresponding lines below.
 */

#ifdef TCL_ACTIVE
#define Tcl_Init Tcl_InitStandAlone
#define Tk_Init Tk_InitStandAlone
#endif

#define Tk_SafeInit (Tcl_PackageInitProc *) NULL

/*
 * Prototypes of all initialization functions and the free() function.
 * So, only "tcl.h" needs to be included now.
 */

#ifdef __cplusplus
extern "C" {
#endif

extern void free _ANSI_ARGS_((void *));
extern int  Tcl_Init _ANSI_ARGS_((Tcl_Interp *interp));
extern int  Tk_Init _ANSI_ARGS_((Tcl_Interp *interp));
#ifndef Tk_SafeInit
extern int  Tk_SafeInit _ANSI_ARGS_((Tcl_Interp *interp));
#endif


#ifdef __cplusplus
}
#endif

/*
 * The array "script" contains the script that is compiled in.
 * It will be executed in tclAppInit() after the other initializations.
 */

static int line = (__LINE__ + 1);
static char *script[] = {
"proc shell {w} {\n\
    text $w\n\
    shellInit $w\n\
    shellPrompt $w \\n\n\
    return $w\n\
}",
"proc shellInit {w} {\n\
    shellBind \n\
    bindtags $w [list Shell $w . all]\n\
}    ",
"proc shellBind {} {\n\
    global tkpriv tk_strictMotif\n\
\n\
    bind Shell <Enter> {}\n\
    bind Shell <FocusIn> {}\n\
    bind Shell <1> {\n\
	tkShellButton1 %W %x %y\n\
	%W tag remove sel 0.0 end\n\
    }\n\
    bind Shell <B1-Motion> {\n\
	set tkPriv(x) %x\n\
	set tkPriv(y) %y\n\
	tkTextSelectTo %W %x %y\n\
    }\n\
    bind Shell <Double-1> {\n\
	set tkPriv(selectMode) word\n\
	tkTextSelectTo %W %x %y\n\
	if [%W compare sel.first > endzone] {\n\
	    %W mark set insert sel.first\n\
	    %W see insert\n\
	}\n\
    }\n\
    bind Shell <Triple-1> {\n\
	set tkPriv(selectMode) line\n\
	tkTextSelectTo %W %x %y\n\
	if [%W compare sel.first > endzone] {\n\
	    %W mark set insert sel.first\n\
	    %W see insert\n\
	}\n\
    }\n\
    bind Shell <ButtonRelease-1> {\n\
	tkCancelRepeat\n\
    }\n\
    bind Shell <Left> {\n\
	shellSetCursor %W [%W index {insert - 1c}]\n\
    }\n\
    bind Shell <Right> {\n\
	shellSetCursor %W [%W index {insert + 1c}]\n\
    }\n\
    bind Shell <Shift-Left> {\n\
	tkTextKeySelect %W [%W index {insert - 1c}]\n\
    }\n\
    bind Shell <Shift-Right> {\n\
	tkTextKeySelect %W [%W index {insert + 1c}]\n\
    }\n\
    bind Shell <Control-Left> {\n\
	shellSetCursor %W [%W index {insert - 1c wordstart}]\n\
    }\n\
    bind Shell <Control-Right> {\n\
	shellSetCursor %W [%W index {insert wordend}]\n\
    }\n\
    bind Shell <Shift-Control-Left> {\n\
	tkTextKeySelect %W [%W index {insert - 1c wordstart}]\n\
    }\n\
    bind Shell <Shift-Control-Right> {\n\
	tkTextKeySelect %W [%W index {insert wordend}]\n\
    }\n\
    bind Shell <Tab> {\n\
	tkTextInsert %W \\t\n\
	focus %W\n\
	break\n\
    }\n\
    bind Shell <Shift-Tab> {\n\
	# Needed only to keep <Tab> binding from triggering;  doesn't\n\
	# have to actually do anything.\n\
    }\n\
    bind Shell <Control-Tab> {\n\
	focus [tk_focusNext %W]\n\
    }\n\
    bind Shell <Control-Shift-Tab> {\n\
	focus [tk_focusPrev %W]\n\
    }\n\
    bind Shell <Control-i> {\n\
	tkTextInsert %W \\t\n\
    }\n\
    bind Shell <Return> {\n\
	tkTextInsert %W \\n\n\
    }\n\
    bind Shell <Delete> {\n\
	if {[%W tag nextrange sel 1.0 end] != \"\"} {\n\
	    %W delete sel.first sel.last\n\
	} else {\n\
	    %W delete insert\n\
	    %W see insert\n\
	}\n\
    }\n\
    bind Shell <BackSpace> {\n\
	if {[%W tag nextrange sel 1.0 end] != \"\"} {\n\
	    %W delete sel.first sel.last\n\
	} elseif [%W compare insert > {endzone + 1 char}] {\n\
	    %W delete insert-1c\n\
	    %W see insert\n\
	}\n\
    }\n\
    bind Shell <Control-space> {\n\
	%W mark set anchor insert\n\
    }\n\
    bind Shell <Select> {\n\
	%W mark set anchor insert\n\
    }\n\
    bind Shell <Control-Shift-space> {\n\
	set tkPriv(selectMode) char\n\
	tkTextKeyExtend %W insert\n\
    }\n\
    bind Shell <Shift-Select> {\n\
	set tkPriv(selectMode) char\n\
	tkTextKeyExtend %W insert\n\
    }\n\
    bind Shell <Insert> {\n\
	catch {tkTextInsert %W [selection get -displayof %W]}\n\
    }\n\
    bind Shell <KeyPress> {\n\
	tkTextInsert %W %A\n\
    }\n\
    bind Shell <Alt-KeyPress> {# nothing }\n\
    bind Shell <Meta-KeyPress> {# nothing}\n\
    bind Shell <Control-KeyPress> {# nothing}\n\
    bind Shell <Escape> {# nothing}\n\
    bind Shell <Control-a> {\n\
	shellSetCursor %W {endzone + 1 char}\n\
    }\n\
    bind Shell <Control-b> {\n\
	shellSetCursor %W insert-1c\n\
    }\n\
    bind Shell <Control-d> {\n\
	%W delete insert\n\
    }\n\
    bind Shell <Control-e> {\n\
	shellSetCursor %W {end - 1 char}\n\
    }\n\
    bind Shell <Control-f> {\n\
	shellSetCursor %W insert+1c\n\
    }\n\
    bind Shell <Control-k> {\n\
	clipboard clear -displayof %W\n\
	clipboard append -displayof %W [%W get insert {end - 1 char}]\n\
	%W delete insert {end - 1 char}\n\
    }\n\
    bind Shell <Control-o> {\n\
	%W insert insert \\n\n\
	%W mark set insert insert-1c\n\
    }\n\
    bind Shell <Meta-b> {\n\
	shellSetCursor %W {insert - 1c wordstart}\n\
    }\n\
    bind Shell <Meta-d> {\n\
	%W delete insert {insert wordend}\n\
    }\n\
    bind Shell <Meta-f> {\n\
	shellSetCursor %W {insert wordend}\n\
    }\n\
    bind Shell <Meta-BackSpace> {\n\
	%W delete {insert -1c wordstart} insert\n\
    }\n\
    bind Shell <Meta-w> {\n\
	if {[selection own -displayof %W] == \"%W\"} {\n\
	    clipboard clear -displayof %W\n\
	    clipboard append -displayof %W [selection get -displayof %W]\n\
	}\n\
    } \n\
    bind Shell <Control-w> {\n\
	if {[selection own -displayof %W] == \"%W\"} {\n\
	    clipboard clear -displayof %W\n\
	    clipboard append -displayof %W [selection get -displayof %W]\n\
	    %W delete sel.first sel.last\n\
	}\n\
    }\n\
    bind Shell <Control-y> {\n\
	catch {\n\
	    %W insert insert [selection get -displayof %W \\\n\
		    -selection CLIPBOARD]\n\
	}\n\
    }\n\
    bind Shell <Control-h> {\n\
	if [%W compare insert != 1.0] {\n\
	    %W delete insert-1c\n\
	    %W see insert\n\
	}\n\
    }\n\
    bind Shell <2> {\n\
	%W scan mark %x %y\n\
	set tkPriv(x) %x\n\
	set tkPriv(y) %y\n\
	set tkPriv(mouseMoved) 0\n\
    }\n\
    bind Shell <B2-Motion> {\n\
	if {(%x != $tkPriv(x)) || (%y != $tkPriv(y))} {\n\
	    set tkPriv(mouseMoved) 1\n\
	}\n\
	if $tkPriv(mouseMoved) {\n\
	    %W scan dragto %x %y\n\
	}\n\
    }\n\
    bind Shell <ButtonRelease-2> {\n\
	if !$tkPriv(mouseMoved) {\n\
	    catch {\n\
		%W insert insert [selection get -displayof %W]\n\
		tkTextSeeInsert %W\n\
	    }\n\
	}\n\
    }\n\
    #rename shellBind {}\n\
    set tkPriv(prevPos) {}\n\
}",
"proc tkShellButton1 {w x y} {\n\
    global tkPriv\n\
\n\
    set tkPriv(selectMode) char\n\
    set tkPriv(mouseMoved) 0\n\
    set tkPriv(pressX) $x\n\
    set pos @$x,$y\n\
    $w mark set anchor $pos\n\
    if [$w compare $pos > endzone] {\n\
	$w mark set insert $pos\n\
	$w see insert\n\
    }\n\
    if {[$w cget -state] == \"normal\"} {focus $w}\n\
}",
"proc shellPrompt {w prompt} {\n\
    global tkPriv\n\
\n\
    $w mark set insert {end - 1 chars}\n\
    $w mark set zone_start {end - 2 chars}\n\
    $w insert insert $prompt\n\
    $w mark set endzone {insert - 1 chars}\n\
    $w tag remove sel 1.0 end\n\
    $w see insert\n\
    $w mark set zone_start {zone_start + 1 chars}\n\
}",
"proc shellPut {w text} {\n\
    $w insert zone_start $text\n\
    $w see insert\n\
}",
"proc shellGet {w} {\n\
    $w get {endzone + 1 chars} {end -1 chars}\n\
}",
"proc shellSet {w text} {\n\
    $w delete {endzone +1 chars} end\n\
    tkTextInsert $w $text\n\
    $w see insert\n\
}",
"    ",
"proc shellSetCursor {w pos} {\n\
    global tkPriv\n\
\n\
    if [$w compare $pos == end] {\n\
	set pos {end - 1 chars}\n\
    }\n\
    if [$w compare $pos <= endzone] {\n\
	set pos {endzone + 1 chars}\n\
    }\n\
    $w mark set insert $pos\n\
    $w tag remove sel 1.0 end\n\
    $w see insert\n\
}",
"proc so_create {type w} {\n\
    set parent [frame $w] \n\
    eval {$type $parent.z -highlightt 0}\n\
    $parent.z config -yscrollcommand [list $parent.sy set] \\\n\
	    -xscrollcommand [list $parent.pad.sx set]\n\
    scrollbar $parent.sy -orient vertical  -takefocus 0 \\\n\
	    -command [list $parent.z yview]\n\
    # create extra frame to hold pad\n\
    frame $parent.pad\n\
    scrollbar $parent.pad.sx -orient horizontal -takefocus 0 \\\n\
	    -command [list $parent.z xview]\n\
    #create padding based on the scrollbars width\n\
    set pad [expr [$parent.sy cget -width] + 2 * \\\n\
	    ([$parent.sy cget -bd] + \\\n\
	    [$parent.sy cget -highlightthickness])]\n\
    frame $parent.pad.it -width $pad -height $pad\n\
    # Arrange everything\n\
    so_plain $parent\n\
    return $parent\n\
}",
"proc so_top_right {w} {\n\
    so_unpack $w\n\
    pack $w.pad -side top -fill x\n\
    pack $w.pad.it -side right\n\
    pack $w.pad.sx -side top -fill x\n\
    pack $w.sy -side right -fill y\n\
    pack $w.z -side left -fill both -expand true    \n\
}",
"proc so_top_left {w} {\n\
    so_unpack $w\n\
    pack $w.pad -side top -fill x\n\
    pack $w.pad.it -side left\n\
    pack $w.pad.sx -side top -fill x\n\
    pack $w.sy -side left -fill y\n\
    pack $w.z -side right -fill both -expand true    \n\
}",
"proc so_bottom_right {w} {\n\
    so_unpack $w\n\
    pack $w.pad -side bottom -fill x\n\
    pack $w.pad.it -side right\n\
    pack $w.pad.sx -side bottom -fill x\n\
    pack $w.sy -side right -fill y\n\
    pack $w.z -side left -fill both -expand true    \n\
}",
"proc so_bottom_left {w} {\n\
    so_unpack $w\n\
    pack $w.pad -side bottom -fill x\n\
    pack $w.pad.it -side left\n\
    pack $w.pad.sx -side bottom -fill x\n\
    pack $w.sy -side left -fill y\n\
    pack $w.z -side right -fill both -expand true    \n\
}",
"proc so_bottom {w} {\n\
    so_unpack $w\n\
    pack $w.pad -side bottom -fill x\n\
    pack $w.pad.sx -side bottom -fill x\n\
    pack $w.z -side left -fill both -expand true    \n\
}",
"proc so_top {w} {\n\
    so_unpack $w\n\
    pack $w.pad -side top -fill x\n\
    pack $w.pad.sx -side top -fill x\n\
    pack $w.z -side left -fill both -expand true    \n\
}",
"proc so_right {w} {\n\
    so_unpack $w\n\
    pack $w.sy -side right -fill y\n\
    pack $w.z -side left -fill both -expand true    \n\
}",
"proc so_left {w} {\n\
    so_unpack $w\n\
    pack $w.sy -side left -fill y\n\
    pack $w.z -side right -fill both -expand true    \n\
}",
"proc so_plain {w} {\n\
    so_unpack $w\n\
    pack $w.z -side left -fill both -expand true\n\
}",
"proc so_unpack {w} {\n\
    pack forget $w.pad.it\n\
    pack forget $w.pad.sx\n\
    pack forget $w.pad\n\
    pack forget $w.sy\n\
    pack forget $w.z\n\
}",
"proc ed_load {w file} {\n\
    if [catch {open $file r} fileId] {\n\
	puts stderr \"$fileId\"\n\
	return -code error\n\
    } else {\n\
	$w delete 1.0 end\n\
	while {![eof $fileId]} {\n\
	    $w insert end [read $fileId 1000]\n\
	}\n\
	close $fileId\n\
    }\n\
}",
"proc ed_save {w file} {\n\
    if [catch {open $file w} fileId] {\n\
	puts stderr \"$fileId\"\n\
	return -code error\n\
    } else {\n\
	puts -nonewline $fileId [$w get 1.0 end]\n\
	close $fileId\n\
    }\n\
}",
"proc mkgrid {canvas colws startrow endrow height font fg bg} {\n\
    set res [list]\n\
    set ncols [llength $colws]\n\
    set y [expr ($startrow-1)*$height+$startrow]\n\
    for {set row $startrow} {$row <= $endrow} {incr row} {\n\
	set x 1\n\
	for {set col 0} {$col < $ncols} {incr col} {\n\
	    set colw [lindex $colws $col]\n\
	    set r [$canvas create re $x $y [expr $x+$colw] [expr $y+$height]\\\n\
		    -f $bg -outline $fg -tag \"q$col\"]\n\
	    set t [$canvas create te [expr $x+2] [expr $y+2]\\\n\
		    -anch nw -fo $font -fi $fg -tag \"q$col\"]\n\
	    $canvas raise $t\n\
	    lappend res $r $t\n\
	    set x [expr $x+$colw+1]\n\
	}\n\
	set y [expr $y+$height+1]\n\
    }\n\
    return $res\n\
}",
"proc calc_new_xs {colws} {\n\
    set ncols [llength $colws]\n\
    set res [list]\n\
    set x 1\n\
    for {set col 0} {$col < $ncols} {incr col} {\n\
	lappend res $x\n\
	set x [expr $x+1+[lindex $colws $col]]\n\
    }\n\
    lappend res $x\n\
    return $res\n\
}",
"proc resize_grid_cols {canvas colws} {\n\
    set item 1\n\
    set xs [calc_new_xs $colws]\n\
    while {[set nbr_of_coords\\\n\
	    [llength [set coords [$canvas coords $item]]]] > 0} {\n\
	set tags [$canvas itemcget $item -tag]\n\
	set first [string first \"q\" $tags]\n\
	set col [string range $tags [expr 1 + $first]\\\n\
		[expr [string wordend $tags $first] -1]]\n\
	switch $nbr_of_coords {\n\
	    2 {  # a text object\n\
		set y [lindex $coords 1]\n\
		set newx [expr [lindex $xs $col] + 2]\n\
		$canvas coords $item $newx $y\n\
	    }\n\
	    4 { # a rectangle object\n\
		set y1 [lindex $coords 1]\n\
		set y2 [lindex $coords 3]\n\
		set newx1 [lindex $xs $col]\n\
		set newx2 [expr [lindex $xs [expr $col + 1]]-1]\n\
		$canvas coords $item $newx1 $y1 $newx2 $y2\n\
	    }\n\
	}\n\
        set item [expr $item+1]\n\
    }\n\
}",
"    ",
"proc erlsend {w1 {w2 \"\"} {w3 \"\"} {w4 \"\"} {w5 \"\"} {w6 \"\"} {w7 \"\"} {w8 \"\"}} {\n\
	set msg [concat $w1 $w2 $w3 $w4 $w5 $w6 $w7 $w8]\n\
	puts -nonewline stdout \"\\1$msg\\5\"\n\
	flush stdout\n\
}",
"proc erlcall {w} {\n\
    set errcode [catch $w result]\n\
    if {$errcode == 0} {\n\
	puts -nonewline stdout \"\\2$result\\5\"\n\
	flush stdout\n\
    } else {\n\
	puts -nonewline stdout \"\\3$result\\5\"\n\
	flush stdout\n\
    }\n\
}",
"proc erlexec {w} {\n\
    set errcode [catch $w result]\n\
    if {$errcode != 0} {\n\
	erlerror $result\n\
    }\n\
}",
"proc erlerror {w} {\n\
    puts -nonewline stdout \"\\4$w\\5\"\n\
    flush stdout\n\
}",
"wm withdraw .",
"set command \"\"",
"fileevent stdin active {\n\
    set line [gets stdin]\n\
    append command \"$line\\n\"\n\
    if [info complete \"${command}\\n\"] {\n\
	if [catch {eval $command} msg] {\n\
	    puts stderr $msg\n\
	} else {\n\
	    if [string compare $msg {}] {\n\
		puts stdout $msg\n\
	    }\n\
	}\n\
	set command \"\"\n\
    }\n\
}",
(char *) NULL
};

/*
 *----------------------------------------------------------------------
 *
 * main --
 *
 *	This is the main program for the application.
 *
 * Results:
 *	None.
 *
 * Side effects:
 *	Whatever the application does.
 *
 *----------------------------------------------------------------------
 */

void
#ifdef _USING_PROTOTYPES_
main (int    argc,		/* Number of command-line arguments. */
      char **argv)		/* Values of command-line arguments. */
#else
main(argc, argv)
    int argc;			/* Number of command-line arguments. */
    char **argv;		/* Values of command-line arguments. */
#endif
{
    Tcl_Interp *interp;
    char **p = script;
    char *q, buffer[10];
    Tcl_DString data;
    Tcl_Channel inChannel, outChannel, errChannel;

    Tcl_FindExecutable(argv[0]);
    interp = Tcl_CreateInterp();

    q = Tcl_Merge(argc-1, argv+1);
    Tcl_SetVar(interp, "argv", q, TCL_GLOBAL_ONLY);
    ckfree(q);
    sprintf(buffer, "%d", argc-1);
    Tcl_SetVar(interp, "argc", buffer, TCL_GLOBAL_ONLY);
    Tcl_SetVar(interp, "argv0", argv[0],TCL_GLOBAL_ONLY);
    Tcl_SetVar(interp, "tcl_interactive","0", TCL_GLOBAL_ONLY);

    if (Tcl_Init(interp) != TCL_OK) {
        goto error;
    }
    Tcl_StaticPackage(interp, "Tk", Tk_Init, Tk_SafeInit);
    if (Tk_Init(interp) != TCL_OK) {
        goto error;
    }

    /*
     * Execute the script that is compiled in.
     */

    inChannel = Tcl_GetStdChannel(TCL_STDIN);
    outChannel = Tcl_GetStdChannel(TCL_STDOUT);
    Tcl_DStringInit(&data);
    while(*p) {
	Tcl_DStringSetLength(&data,0);
	Tcl_DStringAppend(&data,*p++,-1);
	if (Tcl_Eval(interp,Tcl_DStringValue(&data)) != TCL_OK) {
	    Tcl_DStringFree(&data);
	    while (p-- != script) {
		for (q = *p;*q; q++) {
		    if (*q=='\n') line++;
		}
		line++;
	    }
	    sprintf(buffer,"%d",line);
	    Tcl_AddErrorInfo(interp,"\n	( Error in file: \"");
	    Tcl_AddErrorInfo(interp,__FILE__);
	    Tcl_AddErrorInfo(interp,"\", line: ");
	    Tcl_AddErrorInfo(interp,buffer);
	    Tcl_AddErrorInfo(interp,")");
	    errChannel = Tcl_GetStdChannel(TCL_STDERR);
	    if (errChannel) {
		Tcl_Write(errChannel,
			Tcl_GetVar(interp, "errorInfo", TCL_GLOBAL_ONLY), -1);
		Tcl_Write(errChannel, "\n", 1);
	    }
	    sprintf(buffer, "exit %d", 1);
	    Tcl_Eval(interp, buffer);
	}
    }
    Tcl_DStringFree(&data);

    while (Tcl_DoOneEvent(0)) {
	/* empty loop body */ ;
    }
    sprintf(buffer, "exit %d", 0);
    Tcl_Eval(interp, buffer);

error:
    errChannel = Tcl_GetStdChannel(TCL_STDERR);
    if (errChannel) {
	Tcl_Write(errChannel,
		"application-specific initialization failed: ", -1);
	Tcl_Write(errChannel, interp->result, -1);
	Tcl_Write(errChannel, "\n", 1);
    }
    sprintf(buffer, "exit %d", 1);
    Tcl_Eval(interp, buffer);
}
